from __future__ import annotations

from dataclasses import dataclass
from functools import total_ordering
from typing import Tuple


@total_ordering
@dataclass
class Version:
    major: int
    minor: int = 0
    patch: int = 0
    build: int = 0

    @classmethod
    def from_string(cls, string: str) -> Version:
        parts = list(map(int, string.split(".")))

        if len(parts) < 1 or len(parts) > 4:
            raise AssertionError('The version string "{}" is invalid.'.format(string))

        return Version(
            major=parts[0],
            minor=parts[1] if len(parts) > 1 else 0,
            patch=parts[2] if len(parts) > 2 else 0,
            build=parts[3] if len(parts) > 3 else 0,
        )

    def to_tuple(self) -> Tuple[int, int, int, int]:
        return (self.major, self.minor, self.patch, self.build)

    def __lt__(self, other: Version) -> bool:
        return self.to_tuple() < other.to_tuple()

    def __str__(self) -> str:
        return ".".join(map(str, self.to_tuple()))

    def __repr__(self) -> str:
        return "Version({})".format(str(self))



import os
from sys import platform


def extract_chrome_version() -> str:
    if platform in ("linux", "linux2"):
        return _extract_version_from_executable("/usr/bin/google-chrome")
    elif platform == "darwin":
        return _extract_version_from_executable(
            "/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome"
        )
    raise AssertionError('The platform "{}" is not supported.'.format(platform))


def _extract_version_from_executable(executable_path: str) -> str:
    return (
        os.popen("{} --version".format(executable_path))
        .read()
        .strip("Google Chrome ")
        .strip()
    )



import json
import logging
import os
import shutil
import urllib.request
from typing import Iterable, List, Optional

from chromedriver_py_auto.binary_path import binary_path
from chromedriver_py_auto.chrome import extract_chrome_version
from chromedriver_py_auto.version import Version


def copy_binary() -> None:
    import chromedriver_py

    logging.error(
        'Copying "{}" to "{}".'.format(chromedriver_py.binary_path, binary_path)
    )
    shutil.copy2(chromedriver_py.binary_path, binary_path)
    stat = os.stat(chromedriver_py.binary_path)
    os.chown(binary_path, stat.st_uid, stat.st_gid)


def extract_suitable_version() -> Version:
    chrome_version = Version.from_string(extract_chrome_version())
    logging.info('Chrome version "{}" is detected.'.format(str(chrome_version)))

    chromedriver_py_versions = _get_package_versions("chromedriver-py")
    logging.info(
        'Chromedriver "{}" versions are found.'.format(
            '", "'.join(map(str, chromedriver_py_versions))
        )
    )

    most_suitable_chromedriver_py_version = _find_most_suitable_driver_version(
        chrome_version, chromedriver_py_versions
    )
    logging.info(
        'Chromedriver version "{}" is the most suitable for installation.'.format(
            str(most_suitable_chromedriver_py_version)
        )
    )

    return most_suitable_chromedriver_py_version


def _get_package_versions(package_name: str) -> List[Version]:
    url = "https://pypi.org/pypi/%s/json" % (package_name,)
    data = json.load(urllib.request.urlopen(url))
    return sorted(map(Version.from_string, data["releases"]))


def _find_most_suitable_driver_version(
    software_version: Version, driver_versions: Iterable[Version]
) -> Version:
    sorted_driver_versions = sorted(driver_versions)
    pre_driver_version: Optional[Version] = None
    for driver_version in sorted_driver_versions:
        if software_version == driver_version:
            return driver_version
        elif software_version < driver_version:
            if software_version.major == driver_version.major:
                return driver_version
            elif (
                pre_driver_version is not None
                and software_version.major == pre_driver_version.major
            ):
                return pre_driver_version
            break
        pre_driver_version = driver_version

    raise ValueError(
        'For "{}" software version, there is no suitable driver '
        'version in "{}" list.'.format(
            software_version, '", "'.join(map(str, sorted_driver_versions))
        )
    )



from asyncio import subprocess
import logging
from pathlib import Path

import setuptools
from setuptools.command.build_py import build_py

from chromedriver_py_auto.chromedriver_py import copy_binary, extract_suitable_version

logging.basicConfig(level=logging.INFO)


class CopyChromedriverPyBinary(setuptools.command.build_py.build_py):
    def run(self) -> None:
        copy_binary()
        build_py.run(self)


setuptools.setup(
    name="chromedriver-py-auto",
    version="0.2.2",
    description="A wrapper around chromedriver-py library. Detects the Chrome version "
    "and installs the most suitable chromedriver-py version.",
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    license="MIT License",
    author="Alireza Roshanzamir",
    author_email="a.roshanzamir1996@gmail.com",
    url="https://github.com/AlirezaRoshanzamir/chromedriver-py-auto",
    packages=setuptools.find_packages(),
    setup_requires=["chromedriver-py=={}".format(str(extract_suitable_version()))],
    include_package_data=True,
    cmdclass={"build_py": CopyChromedriverPyBinary},
)
